'use client';
import { Fragment as _Fragment, jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Suspense, use, useContext, useRef, useState } from 'react';
import { initializeDevCycle } from '@devcycle/js-client-sdk';
import { useRouter } from 'next/navigation';
import { invalidateConfig } from '../../common/invalidateConfig';
import { DevCycleProviderContext } from './context';
/**
 * Component which renders nothing, but runs code to keep client state in sync with server
 * Also waits for the server's data promise with the `use` hook. This triggers the nearest suspense boundary,
 * so this component is being rendered inside of a Suspense by the DevCycleClientsideProvider.
 * @param serverDataPromise
 * @param userAgent
 * @constructor
 */
export const SuspendedProviderInitialization = ({ serverDataPromise, userAgent, }) => {
    const serverData = use(serverDataPromise);
    const [previousContext, setPreviousContext] = useState();
    const context = useContext(DevCycleProviderContext);
    if (previousContext !== serverData) {
        // change user and config data to match latest server data
        // if the data has changed since the last invocation
        context.client.synchronizeBootstrapData(serverData.config, serverData.user, userAgent);
        setPreviousContext(serverData);
    }
    return _jsx(_Fragment, {});
};
export const InternalDevCycleClientsideProvider = ({ context, children, promiseResolved, }) => {
    const router = useRouter();
    const clientRef = useRef();
    const { serverDataPromise, serverData, clientSDKKey, enableStreaming } = context;
    const revalidateConfig = (lastModified) => {
        invalidateConfig(clientSDKKey, !!context.options.enableObfuscation, lastModified).finally(() => {
            router.refresh();
        });
    };
    let resolvedServerData = serverData;
    if (!serverData && promiseResolved) {
        // here the provider is being told from above that this promise has already resolved, so we can safely "use"
        // it without blocking rendering, even in streaming mode
        resolvedServerData = use(serverDataPromise);
    }
    if (!clientRef.current) {
        clientRef.current = initializeDevCycle(clientSDKKey, {
            ...context.options,
            sdkPlatform: 'nextjs',
            deferInitialization: true,
            disableConfigCache: true,
            next: {
                configRefreshHandler: revalidateConfig,
            },
        });
        if (resolvedServerData || !enableStreaming) {
            // we expect that either the promise has resolved and we got the server data that way, or we weren't in
            // streaming mode and so the promise was awaited at a higher level and passed in here as serverData
            if (!resolvedServerData) {
                throw new Error('Server data should be available. Please contact DevCycle support.');
            }
            clientRef.current.synchronizeBootstrapData(resolvedServerData.config, resolvedServerData.user, context.userAgent);
        }
    }
    return (_jsxs(DevCycleProviderContext.Provider, { value: {
            client: clientRef.current,
            clientSDKKey,
            enableStreaming,
            serverDataPromise,
        }, children: [_jsx(Suspense, { children: _jsx(SuspendedProviderInitialization, { serverDataPromise: serverDataPromise }) }), children] }));
};
//# sourceMappingURL=InternalDevCycleClientsideProvider.js.map